@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2005, 2006, 2007, 2008 Tadashi Watanabe <wac@umiushi.org>
+  Copyright (C) 2005, 2006, 2007, 2008, 2011, 2012, 2013, 2014 Tadashi Watanabe <wac@umiushi.org>
 
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
@@ -16,35 +16,24 @@
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
-
-
-
 #include "skk_utility_architecture.hpp"
 
-
-
-
+namespace YaSkkServ
+{
 namespace SkkUtility
 {
 #ifdef YASKKSERV_CONFIG_ENABLE_ERROR_MESSAGE
-        void printf(const char *p,
-                    ...)
+        void printf(const char *p, ...)
         {
                 va_list ap;
-
-                va_start(ap,
-                         p);
-
-                vfprintf(stderr,
-                         p,
-                         ap);
-
+                va_start(ap, p);
+                vfprintf(stderr, p, ap);
                 va_end(ap);
         }
 #else  // YASKKSERV_CONFIG_ENABLE_ERROR_MESSAGE
-        void printf(const char *p,
-                    ...)
+        void printf(const char *p, ...)
         {
         }
 #endif // YASKKSERV_CONFIG_ENABLE_ERROR_MESSAGE
 }
+}
