@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2005, 2006, 2007, 2008 Tadashi Watanabe <wac@umiushi.org>
+  Copyright (C) 2005, 2006, 2007, 2008, 2011, 2012, 2013, 2014 Tadashi Watanabe <wac@umiushi.org>
 
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
@@ -18,14 +18,10 @@
 #ifndef SKK_COMMAND_LINE_HPP
 #define SKK_COMMAND_LINE_HPP
 
-
-
-
 #include "skk_architecture.hpp"
 
-
-
-
+namespace YaSkkServ
+{
 //
 /**
  *
@@ -49,22 +45,16 @@ class SkkCommandLine
         SkkCommandLine(SkkCommandLine &source);
         SkkCommandLine& operator=(SkkCommandLine &source);
 
-
-
-
 public:
         enum OptionArgument
         {
                 OPTION_ARGUMENT_TERMINATOR,
                 OPTION_ARGUMENT_NONE,
                 OPTION_ARGUMENT_INTEGER,
-//                 OPTION_ARGUMENT_FLOAT,
+                OPTION_ARGUMENT_FLOAT,
                 OPTION_ARGUMENT_STRING
         };
 
-
-
-
         struct Option
         {
                 const char *option_short;
@@ -72,15 +62,11 @@ public:
                 OptionArgument option_argument;
         };
 
-
-
-
-        ~SkkCommandLine()
+        virtual ~SkkCommandLine()
         {
                 delete[] option_result_;
         }
 
-
         SkkCommandLine() :
                 argv_(0),
                 option_table_(0),
@@ -94,10 +80,7 @@ public:
         {
         }
 
-
-        bool parse(int argc,
-                   const char * const argv[],
-                   const Option *option_table)
+        bool parse(int argc, const char * const argv[], const Option *option_table)
         {
                 argc_ = argc;
                 argv_ = argv;
@@ -156,37 +139,31 @@ public:
                                                              *(argv_ + argument_index_ + i));
                                         }
                                 }
-
                                 return true;
                         }
                 }
                 else
                 {
                         error_string_ = "internal error.";
-
                         return false;
                 }
         }
 
-
         int getArgumentArgvIndex() const
         {
                 return argument_index_;
         }
 
-
         int getArgumentLength() const
         {
                 return argument_length_;
         }
 
-
         bool isArgumentDefined(int n) const
         {
                 if (n >= argument_length_)
                 {
                         DEBUG_ASSERT(0);
-
                         return false;
                 }
                 else
@@ -195,13 +172,11 @@ public:
                 }
         }
 
-
         const char *getArgumentPointer(int n) const
         {
                 if (n >= argument_length_)
                 {
                         DEBUG_ASSERT(0);
-
                         return 0;
                 }
                 else
@@ -210,13 +185,11 @@ public:
                 }
         }
 
-
         bool isOptionDefined(int n) const
         {
                 if (n >= option_table_length_)
                 {
                         DEBUG_ASSERT(0);
-
                         return false;
                 }
                 else
@@ -225,13 +198,11 @@ public:
                 }
         }
 
-
         const char *getOptionArgumentPointer(int n) const
         {
                 if (n >= option_table_length_)
                 {
                         DEBUG_ASSERT(0);
-
                         return 0;
                 }
                 else
@@ -240,61 +211,65 @@ public:
                 }
         }
 
-
         const char *getOptionArgumentString(int n) const
         {
                 const char *p = getOptionArgumentPointer(n);
-
                 if (p)
                 {
                         if ((option_table_ + n)->option_argument != OPTION_ARGUMENT_STRING)
                         {
                                 DEBUG_ASSERT(0);
-
                                 return 0;
                         }
                 }
-
                 return p;
         }
 
-
         int getOptionArgumentInteger(int n) const
         {
                 const char *p = getOptionArgumentPointer(n);
-
                 if (p)
                 {
                         if ((option_table_ + n)->option_argument != OPTION_ARGUMENT_INTEGER)
                         {
                                 DEBUG_ASSERT(0);
-
                                 return 0;
                         }
                 }
 
                 int result;
-                if (!SkkUtility::getInteger(p,
-                                            result))
+                if (!SkkUtility::getInteger(p, result))
                 {
                         DEBUG_ASSERT(0);
                 }
-
                 return result;
         }
 
+        float getOptionArgumentFloat(int n) const
+        {
+                const char *p = getOptionArgumentPointer(n);
+                if (p)
+                {
+                        if ((option_table_ + n)->option_argument != OPTION_ARGUMENT_FLOAT)
+                        {
+                                DEBUG_ASSERT(0);
+                                return 0;
+                        }
+                }
 
-//         float getOptionArgumentFloat(int n) const;
-
+                float result;
+                if (!SkkUtility::getFloat(p, result))
+                {
+                        DEBUG_ASSERT(0);
+                }
+                return result;
+        }
 
         const char *getErrorString() const
         {
                 return error_string_;
         }
 
-
-
-
 private:
         static bool is_option(const char *p)
         {
@@ -308,7 +283,6 @@ private:
                 }
         }
 
-
         static bool is_long_option(const char *p)
         {
                 if ((*(p + 0) == '-') &&
@@ -322,48 +296,45 @@ private:
                 }
         }
 
-
         static const char *get_equal_pointer(const char *p)
         {
                 for (;;)
                 {
                         char c = *p;
-
                         if (c == '\0')
                         {
                                 return 0;
                         }
-
                         if (c == '=')
                         {
                                 return p;
                         }
-
                         ++p;
                 }
         }
 
-
         static bool is_integer(const char *p)
         {
                 int dummy_result;
 
-                return SkkUtility::getInteger(p,
-                                              dummy_result);
+                return SkkUtility::getInteger(p, dummy_result);
         }
 
+        static bool is_float(const char *p)
+        {
+                float dummy_result;
+
+                return SkkUtility::getFloat(p, dummy_result);
+        }
 
 /// '=' or '\0' ǽüʸӤƱʤп֤ޤ
-        static bool compare_option_string(const char *a,
-                                          const char *b)
+        static bool compare_option_string(const char *a, const char *b)
         {
                 for (int i = 0; ;++i)
                 {
-                        if ((*(a + i) == '\0') ||
-                            (*(a + i) == '='))
+                        if ((*(a + i) == '\0') || (*(a + i) == '='))
                         {
-                                if ((*(b + i) == '\0') ||
-                                    (*(b + i) == '='))
+                                if ((*(b + i) == '\0') || (*(b + i) == '='))
                                 {
                                         return true;
                                 }
@@ -379,17 +350,13 @@ private:
                 }
         }
 
-
 /// option  option_table_ ˴ޤޤ뤫Ĵ١ޤޤƤп֤ξꤷޤ
-        bool search_option_table(const char *option,
-                                 int &result_option_table_index,
-                                 bool &result_long_flag) const
+        bool search_option_table(const char *option, int &result_option_table_index, bool &result_long_flag) const
         {
                 if (is_option(option))
                 {
                         bool long_flag = is_long_option(option);
                         const char *p;
-
                         if (long_flag)
                         {
                                 p = option + 2; // skip "--"
@@ -398,18 +365,15 @@ private:
                         {
                                 p = option + 1; // skip "-"
                         }
-
                         if (long_flag)
                         {
                                 for (int i = 0; (option_table_ + i)->option_argument != OPTION_ARGUMENT_TERMINATOR; ++i)
                                 {
                                         if ((option_table_ + i)->option_long &&
-                                            compare_option_string(p,
-                                                                  (option_table_ + i)->option_long))
+                                            compare_option_string(p, (option_table_ + i)->option_long))
                                         {
                                                 result_option_table_index = i;
                                                 result_long_flag = long_flag;
-
                                                 return true;
                                         }
                                 }
@@ -419,22 +383,18 @@ private:
                                 for (int i = 0; (option_table_ + i)->option_argument != OPTION_ARGUMENT_TERMINATOR; ++i)
                                 {
                                         if ((option_table_ + i)->option_short &&
-                                            compare_option_string(p,
-                                                                  (option_table_ + i)->option_short))
+                                            compare_option_string(p, (option_table_ + i)->option_short))
                                         {
                                                 result_option_table_index = i;
                                                 result_long_flag = long_flag;
-
                                                 return true;
                                         }
                                 }
                         }
                 }
-
                 return false;
         }
 
-
         bool parse_loop(int option_check_length)
         {
                 enum State
@@ -455,37 +415,39 @@ private:
                                      *(argv_ + i));
                         switch (state)
                         {
+                        default:
+                                DEBUG_ASSERT(0);
+                                error_string_ = "internal error.";
+                                return false;
                         case STATE_OPTION:
-                                ;
                                 DEBUG_PRINTF("    state_option\n");
                                 {
                                         bool long_flag;
-                                        if (search_option_table(*(argv_ + i),
-                                                                option_table_index,
-                                                                long_flag))
+                                        if (search_option_table(*(argv_ + i), option_table_index, long_flag))
                                         {
                                                 DEBUG_PRINTF("    search_option_table() found.\n");
 
                                                 switch ((option_table_ + option_table_index)->option_argument)
                                                 {
+                                                case OPTION_ARGUMENT_TERMINATOR: // FALLTHROUGH
+                                                default:
+                                                        DEBUG_ASSERT(0);
+                                                        error_string_ = "internal error.";
+                                                        return false;
                                                 case OPTION_ARGUMENT_NONE:
-                                                        ;
                                                         {
                                                                 const char *p = get_equal_pointer(*(argv_ + i));
                                                                 if (p)
                                                                 {
                                                                         error_string_ = p;
-
                                                                         return false;
                                                                 }
                                                         }
                                                         (option_result_ + option_table_index)->enable_flag = true;
                                                         break;
-
                                                 case OPTION_ARGUMENT_INTEGER:
-//                                         case OPTION_ARGUMENT_FLOAT:
+                                                case OPTION_ARGUMENT_FLOAT:
                                                 case OPTION_ARGUMENT_STRING:
-                                                        ;
                                                         {
                                                                 const char *p = get_equal_pointer(*(argv_ + i));
                                                                 if (p)
@@ -496,7 +458,6 @@ private:
                                                                         if (*(p + 1) == '\0')
                                                                         {
                                                                                 error_string_ = p;
-
                                                                                 return false;
                                                                         }
                                                                         if ((option_table_ + option_table_index)->option_argument == OPTION_ARGUMENT_INTEGER)
@@ -504,7 +465,14 @@ private:
                                                                                 if (!is_integer(p + 1))
                                                                                 {
                                                                                         error_string_ = p;
-
+                                                                                        return false;
+                                                                                }
+                                                                        }
+                                                                        if ((option_table_ + option_table_index)->option_argument == OPTION_ARGUMENT_FLOAT)
+                                                                        {
+                                                                                if (!is_float(p + 1))
+                                                                                {
+                                                                                        error_string_ = p;
                                                                                         return false;
                                                                                 }
                                                                         }
@@ -519,12 +487,6 @@ private:
                                                                 }
                                                         }
                                                         break;
-
-                                                default:
-                                                        DEBUG_ASSERT(0);
-                                                        error_string_ = "internal error.";
-
-                                                        return false;
                                                 }
                                         }
                                         else
@@ -546,13 +508,11 @@ private:
                                         }
                                 }
                                 break;
-
                         case STATE_OPTION_ARGUMENT:
                                 DEBUG_PRINTF("    state_option_argument\n");
                                 if (option_table_index < 0)
                                 {
                                         error_string_ = "internal error.";
-
                                         return false;
                                 }
                                 if ((option_table_ + option_table_index)->option_argument == OPTION_ARGUMENT_INTEGER)
@@ -560,7 +520,14 @@ private:
                                         if (!is_integer(*(argv_ + i)))
                                         {
                                                 error_string_ = *(argv_ + i);
-
+                                                return false;
+                                        }
+                                }
+                                if ((option_table_ + option_table_index)->option_argument == OPTION_ARGUMENT_FLOAT)
+                                {
+                                        if (!is_float(*(argv_ + i)))
+                                        {
+                                                error_string_ = *(argv_ + i);
                                                 return false;
                                         }
                                 }
@@ -568,30 +535,17 @@ private:
                                 (option_result_ + option_table_index)->enable_flag = true;
                                 state = STATE_OPTION;
                                 break;
-
-                        default:
-                                DEBUG_ASSERT(0);
-                                error_string_ = "internal error.";
-
-                                return false;
                         }
                 }
-
                 return true;
         }
 
-
-
-
 private:
         struct OptionResult
         {
                 OptionResult(OptionResult &source);
                 OptionResult& operator=(OptionResult &source);
 
-
-
-
                 OptionResult () :
                         argument(0),
                         enable_flag(false),
@@ -599,17 +553,11 @@ private:
                 {
                 }
 
-
-
-
                 const char *argument;
                 bool enable_flag;
                 bool long_flag;
         };
 
-
-
-
         const char * const *argv_;
         const Option *option_table_;
         OptionResult *option_result_;
@@ -620,8 +568,6 @@ private:
         int argument_length_;
         bool parse_flag_;
 };
-
-
-
+}
 
 #endif // SKK_COMMAND_LINE_HPP
